/*****************************************************************************
 This is a UDP example Java application to send and receive UDP datagrams.
 It will listen on a port for incoming UDP packets, and allow the user to
 send strings as UDP datagrams to another host.

 This application can run on Windows, Unix or Linux hosts with the Java
 Virtual Machine installed. It is not designed to run on the NetBurner
 platform.

 Revision 1.0, May 31, 2003

*****************************************************************************/

import java.net.*;
import java.io.*;

/*----------------------------------------------------------------
 UDP Server Class
 This class will:
 1. Parse command line parameters for the local port number,
    destination ip address, and destination port number.

 2. Obtain the local host name and ip address, then display all
    parameters to the console.

 3. Declare an instance of the client UDP class, which will
    handle local host user input and allow the user to send
	 strings.

 4. Start the UDP server thread that will loop forever listening
    for incoming UDP packets on the designated port. All 
	 incoming messages will be displayed to the console window
	 along with the sender's ip address and port number.
----------------------------------------------------------------*/
public class udpserv {
		protected DatagramSocket dgSocket;	// declare socket

		// Constructor
		public udpserv (int port) throws IOException {
			dgSocket = new DatagramSocket(port);    // allocate socket
		}
	
	// execute() loop to wait for incoming datagram packets
	public void execute()throws IOException {
		while (true) 
		{
			byte buffer[] = new byte[65536];
			DatagramPacket dgPacket = new DatagramPacket(buffer, buffer.length);
			dgSocket.receive(dgPacket);

			System.out.print("Rec'd From: " + dgPacket.getAddress());
			System.out.print(":" + dgPacket.getPort());
			System.out.print(", " + dgPacket.getLength() + " bytes: \"");
			String msg = new String(buffer, 0, dgPacket.getLength());
			System.out.println(msg + "\"\n");
		}
	}


	public static void main(String args[]) throws IOException {
		int ListenPort, DestPort;
		String DestIP;

		if (args.length != 3) 
		   throw new RuntimeException("Syntax:  udpserv <listen port> <dest IP> <dest port>");
		ListenPort = Integer.parseInt(args[0]);
		DestIP     = args[1];
		DestPort   = Integer.parseInt(args[2]);

		// Display command line parameters
		System.out.println("Listen Port: " + ListenPort);
		System.out.println("Dest IP    : " + DestIP);
		System.out.println("Dest Port  : " + DestPort);

		InetAddress LocalHost = InetAddress.getLocalHost();
		System.out.println("Local Host Name: " + LocalHost.getHostName());
		System.out.println("Local Host IP Addr: " + LocalHost.getHostAddress());

		// Create instance of udpserv class
		udpserv serverUDP = new udpserv(ListenPort);
		System.out.println("Starting UDP echo server, listening on port " + ListenPort);


		// Convert destination IP string to InetAddress type
		InetAddress DestAddr = InetAddress.getByName(DestIP);

		// Start thread to accept user input strings and send them
		// as UDP datagrams. This is the client side of this example
		clientUDPThread ct = new clientUDPThread(DestAddr, DestPort);
		ct.start();

		// Start server execution to receive UDP datagrams
		serverUDP.execute();
	}
}	



/*-------------------------------------------------------------------
 UDP Client Class
 This class simply waits for user input from the console, creates
 a UDP datagram with the input data, and sends the datagram to
 the designated destination host.
 -------------------------------------------------------------------*/
class clientUDPThread extends Thread {

	int DestPort;
	InetAddress DestAddr;
	protected DatagramSocket dgSocket;	// declare socket

	public clientUDPThread(InetAddress DestinationAddr, int DestinationPort) throws IOException
	{
		dgSocket = new DatagramSocket(DestPort);    // allocate socket
		DestAddr = DestinationAddr;
		DestPort = DestinationPort;
	}

	public String GetUserInput()
	{
		String s = "";

		try {
			while (true) {
				char c = (char)System.in.read();
				if (c == '\r') {
					c = (char)System.in.read();
					if (c == '\n') {
						break;
					} else {
						continue;
					}
				} else if (c == '\n') {
					break;
				} else {
					s += c;		// store char
				}
			} // while

		}
		catch (Exception e) {
			System.err.println("Exception: " + e.toString());
		}
		return s;
	}


	public void run() 
	{
		while (true)
		{
			String msg = GetUserInput();
			byte[] buffer = msg.getBytes();
			System.out.println("Sending Message: \"" + msg + "\"");
			DatagramPacket packet = new DatagramPacket(buffer, 
												buffer.length, DestAddr, DestPort);
			try {
				dgSocket.send(packet);
			} catch (Exception e) {
				System.err.println("Exception: " + e.toString());
			}
		}
	}


}


